import React, {Component} from 'react'
import PropTypes from 'prop-types'
import moment from 'moment'
import {connect} from 'react-redux'
import { is, fromJS } from 'immutable'
import { Button, Modal, notification, message } from 'antd'

import Api from '@/api'
import Utils from '@/utils/utils.js'
import options from '@/store/options.js'
import zhCN from '@/locales/zh-CN/main.js'
import enUS from '@/locales/en-US/main.js'
import asyncSpinComponent from '@/utils/asyncSpinComponent'
import MKEmitter from '@/utils/events.js'
import './index.scss'

const MutilForm = asyncSpinComponent(() => import('@/tabviews/zshare/mutilform'))
const { confirm } = Modal
let socket = null

class PrintButton extends Component {
  static propTpyes = {
    show: PropTypes.any,              // 按钮显示样式控制
    position: PropTypes.any,          // 按钮位置，工具栏为toolbar
    BID: PropTypes.string,            // 主表ID
    BData: PropTypes.any,             // 主表数据
    selectedData: PropTypes.any,      // 子表中选择数据
    Tab: PropTypes.any,               // 如果当前元素为标签时，tab为标签信息
    MenuID: PropTypes.string,         // 菜单ID
    btn: PropTypes.object,            // 按钮
    setting: PropTypes.any,           // 页面通用设置
    ContainerId: PropTypes.any,       // tab页面ID，用于弹窗控制
  }

  state = {
    dict: sessionStorage.getItem('lang') !== 'en-US' ? zhCN : enUS,
    visible: false,
    formdata: null,
    tabledata: null,
    btnconfig: null,
    loading: false,
    loadingNumber: ''
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props), fromJS(nextProps)) || !is(fromJS(this.state), fromJS(nextState))
  }

  componentDidMount () {
    const { position } = this.props

    if (position === 'toolbar') {
      MKEmitter.addListener('triggerBtnId', this.actionTrigger)
    }
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('triggerBtnId', this.actionTrigger)
  }

  /**
   * @description 按钮状态改变
   */
  updateStatus = (type) => {
    if (type === 'start') {
      this.setState({
        loading: true
      })
    } else if (type === 'over') {
      this.setState({
        loading: false
      })
    }
  }
  
  /**
   * @description 触发按钮操作
   */
  actionTrigger = (triggerId, record) => {
    const { setting, Tab, BID, btn, selectedData } = this.props
    const { loading } = this.state

    if ((triggerId && btn.uuid !== triggerId) || loading) return

    if (Tab && Tab.supMenu && !BID) {
      notification.warning({
        top: 92,
        message: '需要上级主键值！',
        duration: 5
      })
      return
    }

    let _this = this
    let data = record || selectedData || []

    if (btn.Ot !== 'notRequired' && data.length === 0) {
      // 需要选择行时，校验数据
      notification.warning({
        top: 92,
        message: this.state.dict['main.action.confirm.selectline'],
        duration: 5
      })
      return
    } else if (btn.Ot === 'requiredSgl' && data.length !== 1) {
      // 需要选择单行时，校验数据
      notification.warning({
        top: 92,
        message: this.state.dict['main.action.confirm.selectSingleLine'],
        duration: 5
      })
      return
    } else if (!setting.primaryKey) {
      // 需要选择行时，校验是否设置主键
      notification.warning({
        top: 92,
        message: '未设置主键！',
        duration: 5
      })
      return
    } else if (!['requiredSgl', 'notRequired', 'requiredOnce', 'required'].includes(btn.Ot)) {
      // 数据选择类型校验
      this.actionSettingError()
      return
    } else if (
      !btn.verify ||
      (btn.verify.printMode === 'normal' && (!btn.verify.Template || !btn.verify.linkUrl)) ||
      (btn.verify.printMode === 'custom' && (!btn.verify.printFunc || !btn.verify.linkUrl))
    ) {
      notification.warning({
        top: 92,
        message: '请完善打印验证信息！',
        duration: 5
      })
      return
    }

    if (btn.execMode === 'pop') {
      this.updateStatus('start')
      let modal = this.state.btnconfig
      if (!modal && btn.modal) {
        modal = this.handleModelConfig(btn.modal)
      }

      this.setState({
        tabledata: data,
        btnconfig: modal
      }, () => {
        this.improveAction()
      })
    } else if (btn.execMode === 'prompt') {
      this.updateStatus('start')
      confirm({
        title: this.state.dict['main.action.confirm.tip'],
        onOk() {
          _this.triggerPrint(data)
        },
        onCancel() {
          _this.updateStatus('over')
        }
      })
    } else {
      this.triggerPrint(data)
    }

    if (window.GLOB.systemType === 'production') {
      MKEmitter.emit('queryTrigger', {menuId: btn.uuid, name: '标签打印'})
    }
  }

  /**
   * @description 触发打印
   */
  triggerPrint = (data, formlist = []) => {
    const { btn } = this.props
    let formdata = {}
    
    formlist.forEach(_data => {
      formdata[_data.key] = _data.value
    })

    let printlist = []
    let templates = []
    let printCount = 1

    if (formdata.printCount && typeof(formdata.printCount) === 'number' && formdata.printCount >= 1) {
      printCount = formdata.printCount
    }

    new Promise(resolve => {
      if (btn.intertype === 'system') { // 使用系统时，直接从表格或表单中选取数据
        let printcell = {}

        printcell.printType = formdata.printType || ''
        printcell.printCount = printCount
        printcell.templateID = btn.verify.Template || ''

        if (btn.Ot === 'notRequired') {
          printcell.data = [formdata]
        } else {
          printcell.data = data.map(cell => {
            return {...cell, ...formdata}
          })
        }

        templates.push(printcell.templateID)

        printlist.push(printcell)

        resolve(true)
      } else {
        this.getprintdata(btn, data, formdata, formlist).then(result => {
          if (result.next) {
            result.list.forEach(cell => {
              // 系统打印数据，校验data字段
              if (btn.verify.printMode !== 'custom' && (!cell.data || cell.data.length === 0)) return

              cell.templateID = cell.templateID || btn.verify.Template
              cell.printType = cell.printType || formdata.printType || ''

              let _printCount = printCount
              if (cell.printCount && typeof(cell.printCount) === 'number' && cell.printCount >= 1) {
                _printCount = cell.printCount
              }
              
              cell.printCount = _printCount

              templates.push(cell.templateID)

              printlist.push(cell)
            })
          }
          
          resolve(result.next)
        })
      }
    }).then(res => {
      // 获取打印模板 getTemp
      if (!res) return false

      if (btn.verify.printMode === 'custom') {
        this.execCustomPrint(printlist, formdata)

        return false
      }

      templates = Array.from(new Set(templates)) // 去重

      let deffers = templates.map(tempId => {
        return new Promise(resolve => {
          let param = {
            func: 's_PrintTemplateMGetData',
            Type: 'Y',
            // ID: tempId, // 添加模板时，保存及查询使用模板参数
            PrintTempNO: tempId
          }
    
          if (window.GLOB.mainSystemApi) { // 从单点登录服务器取打印配置信息
            param.rduri = window.GLOB.mainSystemApi
          }
    
          Api.getLocalConfig(param).then(result => {
            result.tempId = tempId
            resolve(result)
          })
        })
      })

      return Promise.all(deffers)
    }).then(result => {
      if (!result) return

      let errorMsg = ''
      let _temps = {}

      result.forEach(res => {
        if (res.status && !errorMsg) {
          let _temp = this.getPrintConfigParam(res)

          if (_temp.error) {
            errorMsg = {
              ErrCode: 'N',
              message: _temp.error,
              ErrMesg: _temp.error,
              status: false
            }
          } else {
            _temps[res.tempId] = _temp
          }
        } else if (!errorMsg) {
          errorMsg = res
        }
      })

      if (!errorMsg) {
        this.execPrint(printlist, _temps, formdata)
      } else {
        this.execError(errorMsg)
      }
    })
  }

  /**
   * @description 自定义打印，解析自定义js
   */
  execCustomPrint = (printlist, formdata) => {
    const { btn } = this.props

    this.execSuccess({
      ErrCode: '-1',
      message: '',
      ErrMesg: '',
      status: true
    })

    try {
      // eslint-disable-next-line
      let func = new Function('data', 'form', 'printer', 'notification', btn.verify.printFunc)
      func(printlist, formdata, btn.verify, notification)
    } catch (e) {
      console.warn(e)

      try {
        // eslint-disable-next-line
        let evalfunc = eval('(true && function (data, form, printer, notification) {' + btn.verify.printFunc + '})')
        evalfunc(printlist, formdata, btn.verify, notification)
      } catch (error) {
        console.warn(error)
        notification.warning({
          top: 92,
          message: '自定义函数执行错误！',
          duration: 5
        })
      }
    }
  }

  /**
   * @description 获取打印数据
   */
  getprintdata = (btn, data, formdata, formlist) => {
    const { setting } = this.props

    let _list = []
    return new Promise(resolve => {
      let params = []
      let param = {}

      if (this.props.BID) {
        param.BID = this.props.BID
      }

      if (btn.Ot === 'notRequired') {
        let _param = { ...param, ...formdata }
        params.push(_param)
      } else if (btn.Ot === 'requiredSgl') {
        param[setting.primaryKey] = data[0][setting.primaryKey]

        let _param = { ...param, ...formdata }

        params.push(_param)
      } else if (btn.Ot === 'requiredOnce') {
        let ids = data.map(d => { return d[setting.primaryKey]})
        ids = ids.filter(Boolean)
        ids = ids.join(',')

        param[setting.primaryKey] = ids

        let _param = { ...param, ...formdata }

        params.push(_param)
      } else if (btn.Ot === 'required') {
        params = data.map((cell, index) => {
          let _param = { ...param }
          _param[setting.primaryKey] = cell[setting.primaryKey]

          formlist.forEach(_data => {
            if (index !== 0 && _data.readin && cell.hasOwnProperty(_data.key)) {
              _param[_data.key] = cell[_data.key]
            } else {
              _param[_data.key] = _data.value
            }
          })
          return _param
        })
      }

      if (btn.intertype === 'inner') {
        params = params.map(_param => {
          _param.func = btn.innerFunc

          return _param
        })

        if (params.length <= 20) {
          let deffers = params.map(par => {
            return new Promise(resolve => {
              Api.genericInterface(par).then(res => {
                resolve(res)
              })
            })
          })
          Promise.all(deffers).then(result => {
            let errorMsg = ''
            result.forEach(res => {
              if (!res.status) {
                errorMsg = res
              }
            })
            if (!errorMsg) {
              resolve({next: true, list: result})
            } else {
              this.execError(errorMsg)
              resolve({next: false, list: []})
            }
          })
        } else {
          this.printInnerLoopRequest(params, btn, _list, resolve)
        }
      } else {
        this.printOuterLoopRequest(params, btn, _list, resolve)
      }
    })
  }

  /**
   * @description 外部请求循环执行
   */
  printOuterLoopRequest = (params, btn, _list, _resolve) => {
    let param = params.shift()
    let _outParam = null

    new Promise(resolve => {
      // 内部请求
      if (btn.innerFunc) {
        param.func = btn.innerFunc
        // 存在内部函数时，数据预处理
        Api.genericInterface(param).then(res => {
          if (res.status) {
            delete res.ErrCode
            delete res.ErrMesg
            delete res.message
            delete res.status

            // 使用处理后的数据调用外部接口
            let keys = Object.keys(res) // 提交外部接口前，添加BID
            if (this.props.BID && keys.filter(key => key.toLowerCase() === 'bid').length === 0) {
              res.BID = this.props.BID
            }
            
            resolve(res)
          } else {
            this.execError(res)
            resolve(false)
            _resolve({next: false, list: []})
          }
        })
      } else {
        resolve(param)
      }
    }).then(res => {
      if (!res) return
      // 外部请求
      _outParam = JSON.parse(JSON.stringify(res))

      if (this.props.menuType === 'HS') {
        if (btn.sysInterface === 'true' && options.cloudServiceApi) {
          res.rduri = options.cloudServiceApi
        } else if (btn.sysInterface !== 'true') {
          if (window.GLOB.systemType === 'production' && btn.proInterface) {
            res.rduri = btn.proInterface
          } else {
            res.rduri = btn.interface
          }
        }
      } else {
        if (btn.sysInterface === 'true' && window.GLOB.mainSystemApi) {
          res.rduri = window.GLOB.mainSystemApi
        } else if (btn.sysInterface !== 'true') {
          if (window.GLOB.systemType === 'production' && btn.proInterface) {
            res.rduri = btn.proInterface
          } else {
            res.rduri = btn.interface
          }
        }
      }

      if (btn.outerFunc) {
        res.func = btn.outerFunc
      }

      return Api.genericInterface(res)
    }).then(response => {
      if (!response) return

      if (btn.callbackFunc) {
        // 存在回调函数时，调用
        delete response.message
        delete response.status

        response.func = btn.callbackFunc

        let _callbackparam = {..._outParam, ...response}

        return Api.genericInterface(_callbackparam)
      } else if (response.status) {

        _list.push(response)

        // 一次请求成功，进行下一项请求
        if (params.length === 0) {
          _resolve({next: true, list: _list})
        } else {
          this.printOuterLoopRequest(params, btn, _list, _resolve)
        }
      } else {
        this.execError(response)
        _resolve({next: false, list: []})
      }
    }).then(response => {
      if (!response) return

      if (response.status) {
        _list.push(response)

        // 一次请求成功，进行下一项请求
        if (params.length === 0) {
          _resolve({next: true, list: _list})
        } else {
          this.printOuterLoopRequest(params, btn, _list, _resolve)
        }
      } else {
        this.execError(response)
        _resolve({next: false, list: []})
      }
    })
  }

  /**
   * @description 内部请求循环执行
   */
  printInnerLoopRequest = (params, btn, _list, _resolve) => {
    let param = params.shift()

    Api.genericInterface(param).then(res => {
      if (res.status) {
        _list.push(res)

        if (params.length === 0) {
          _resolve({next: true, list: _list})
        } else {
          this.printInnerLoopRequest(params, btn, _list, _resolve)
        }
      } else {
        this.execError(res)
        _resolve({next: false, list: []})
      }
    })
  }

  getPrintConfigParam = (res) => {
    let error = ''         // 错误信息
    let configParam = ''   // 模板配置信息
    let _configparam = ''  // 打印配置信息
    let fields = []        // 模板中所需字段
    let nonEFields = []    // 非空字段

    if (!res.ConfigParam) {
      error = '未获取到打印模板信息！'
    } else {
      try {
        configParam = JSON.parse(window.decodeURIComponent(window.atob(res.ConfigParam)))
      } catch (e) {
        configParam = ''
      }

      if (!configParam) {
        error = '打印模板解析错误！'
      } else {
        
        let control = configParam.elements.map(element => {
          let _field = element.field

          if (_field === 'other_field') {
            _field = element.cusfield || ''
          }

          let item = {
            Name: element.name || '',
            Type: element.type,
            Value: element.value || '',
            Field: _field,
            Left: element.left,
            Top: element.top,
            Width: element.width,
            Height: element.height,
            Rotate: element.rotate,
            BorderSize: element.borderSize / 10,
            BorderColor: element.borderColor,
            Align: element.align,
            VerticalAlign: element.vertialAlign,
            BackColor: element.background,
            ForeColor: 'black'
          }
  
          if (!item.Width || !item.Height) {
            item.Type = 'line'
            item.Value = ''
            item.Field = ''
            item.FontFamily = element.fontFamily || ''
            item.FontSize = element.fontSize || ''
            item.FontStyle = (!element.fontWeight || element.fontWeight === 'normal') ? 'regular' : element.fontWeight
            item.Padding = 0
            item.Trimming = ''
            if (!item.Width) {
              item.Width = item.BorderSize
              item.Left = item.Left - item.Width
            } else if (!item.Height) {
              item.Height = item.BorderSize
              item.Top = item.Top - item.Height
            }
            item.BorderSize = 0
          } else if (item.Type === 'image') {
            item.ImageWidth = element.imgWidth
            item.ImageHeight = element.imgHeight
            item.Trimming = ''
          } else if (item.Type === 'text') {
            item.FontFamily = element.fontFamily
            item.FontSize = element.fontSize
            item.FontStyle = element.fontWeight === 'normal' ? 'regular' : element.fontWeight
            item.Padding = element.padding / 2
            item.Trimming = ''
          } else if (item.Type === 'barcode') {
            item.BarcodeType = element.barcodeType
            item.BarcodeWidth = element.barcodeWidth
            item.BarcodeHeight = element.barcodeHeight
            item.BarcodeLabel = element.barcodeLabel
            item.LabelSize = element.fontSize
          } else if (item.Type === 'qrcode') {
            item.Type = 'barcode'
            item.BarcodeType = element.qrcodeType
            item.BarcodeWidth = element.qrcodeWidth
            item.BarcodeHeight = element.qrcodeWidth
            item.BarcodeLabel = false
          }

          if (item.Field) {
            fields.push(item.Field)
            // 条码二维码字段不可为空
            if (item.Type === 'qrcode' || item.Type === 'barcode') {
              nonEFields.push(item.Field)
            }
          }

          return item
        })

        _configparam = {
          Version: '',
          Title: configParam.name,
          Author: sessionStorage.getItem('UserID'),
          Description: configParam.remark,
          PrintTempNO: configParam.PrintTempNO,
          PageSetting: {
            Width: configParam.width,
            Height: configParam.height,
            Left: '0',
            Right: '0',
            Top: '0',
            Bottom: '0',
            Landscape: false
          },
          PageHeader: [],
          ReportHeader: {
            Control: control
          },
          ReportFooter: [],
          PageFooter: []
        }
      }
    }

    return {
      error: error,
      config: _configparam,
      fields: fields,
      nonEFields: nonEFields
    }
  }

  execPrint = (list, template, formdata) => {
    const { btn } = this.props
    let _errors = []
    
    let defaultPrinter = btn.verify.defaultPrinter || 'lackprinter'
    let printers = {}
    if (btn.verify.printerTypeList && btn.verify.printerTypeList.length > 0) {
      btn.verify.printerTypeList.forEach(cell => {
        if (cell.printer) {
          printers[cell.Value] = cell.printer
        }
      })
    }

    let printdata = {}

    list.forEach(res => {
      let _printer = defaultPrinter

      if (res.printType && printers[res.printType]) {
        _printer = printers[res.printType]
      }

      printdata[_printer] = printdata[_printer] || []

      printdata[_printer].push(res)
    })

    let printerList = []

    Object.keys(printdata).forEach(printer => {
      Object.keys(template).forEach(key => {
        let _datalist = printdata[printer].filter(cell => cell.templateID === key)

        if (_datalist.length === 0) return

        let _data = []
        _datalist.forEach(res => {
          res.data.forEach(_cell => {
            for (let i = 0; i < res.printCount; i++) {
              _data.push({...formdata, ..._cell})
            }
          })
        })

        let _fields = Array.from(new Set(template[key].fields))
        let _nonEFields = Array.from(new Set(template[key].nonEFields))
        let lacks = []
        let emptys = []

        _data.forEach(d => {
          _fields.forEach(f => {
            if (!d.hasOwnProperty(f)) {
              lacks.push(f)
            } else if (_nonEFields.includes(f) && !d[f] && d[f] !== 0) {
              emptys.push(f)
            }
          })
        })

        if (lacks.length > 0 || emptys.length > 0) {
          lacks = Array.from(new Set(lacks))
          emptys = Array.from(new Set(emptys))

          _errors.push({
            title: template[key].config.Title,
            lacks: lacks,
            emptys: emptys
          })
        }

        let results = []
        let num = 100
        for(let i = 0, len = _data.length; i < len; i += num){
          results.push(_data.slice(i, i + num))
        }

        results.forEach(result => {
          let _cell = {
            documentID: Utils.getuuid(),
            contents: [
              {
                data: result,
                templateURL: JSON.stringify(template[key].config)
              }
            ]
          }
  
          printerList.push({
            cmd: 'print',
            requestID: Utils.getuuid(),
            version: Utils.getuuid(),
            task: {
              taskID: Utils.getuuid(),
              preview: false,
              printer: printer,
              documents: [_cell]
            }
          })
        })
      })
    })

    if (list.length === 0) {
      this.execError({
        ErrCode: 'N',
        message: '未获取到打印信息！',
        ErrMesg: '',
        status: false
      })
      return
    } else if (_errors.length > 0) {
      let lackerror = []
      let emptyerror = []

      _errors.forEach(err => {
        if (err.lacks.length > 0) {
          lackerror.push(`数据中未获取到模板（${err.title}）${err.lacks.join('、')} 字段`)
        }
        if (err.emptys.length > 0) {
          emptyerror.push(`数据中模板（${err.title}） ${err.emptys.join('、')} 字段不可为空`)
        }
      })

      let msg = []
      if (lackerror.length > 0) {
        msg.push(lackerror.join(' ; '))
      }

      if (emptyerror.length > 0) {
        msg.push(emptyerror.join(' ; '))
      }

      this.execError({
        ErrCode: 'N',
        message: msg.join(' ; ') + ' !',
        ErrMesg: '',
        status: false
      })
      return
    }

    let lackItems = printerList.filter(cell => cell.task.printer === 'lackprinter')[0]

    if (!socket || socket.readyState !== 1 || socket.url !== 'ws://' + btn.verify.linkUrl) {
      socket = new WebSocket('ws://' + btn.verify.linkUrl)
    } else {
      if (lackItems) {
        let request  = {
          requestID: '',
          version: '',
          cmd: 'getPrinters'
        }
        socket.send(JSON.stringify(request))
      } else {
        this.syncMessageSend(printerList)

        this.execSuccess({
          ErrCode: 'S',
          message: '',
          ErrMesg: '打印请求已发出。',
          status: true
        })
      }
    }

    // 打开Socket
    socket.onopen = () =>{
      if (lackItems) {
        let request  = {
          requestID: '',
          version: '',
          cmd: 'getPrinters'
        }
        socket.send(JSON.stringify(request))
      } else {
        this.syncMessageSend(printerList)

        this.execSuccess({
          ErrCode: 'S',
          message: '',
          ErrMesg: '打印请求已发出。',
          status: true
        })
      }
    }
    // 监听消息
    socket.onmessage = (event) => {
      let data = ''

      if (event.data) {
        try {
          data = JSON.parse(event.data)
        } catch {
          this.execError({
            ErrCode: 'N',
            message: event.data,
            ErrMesg: '',
            status: false
          })

          data = ''
        }
      }

      if (data && data.cmd === 'getPrinters' && data.status) {
        printerList = printerList.map(cell => {
          if (cell.task.printer === 'lackprinter') {
            cell.task.printer = data.defaultPrinter
          }
          return cell
        })

        this.syncMessageSend(printerList)

        this.execSuccess({
          ErrCode: 'S',
          message: '',
          ErrMesg: '打印请求已发出。',
          status: true
        })
      } else if (data && data.message && !data.status) {
        this.execError({
          ErrCode: 'N',
          message: data.message,
          ErrMesg: '',
          status: false
        })
      }
    }

    socket.onerror = () => {
      this.execError({
        ErrCode: 'N',
        message: '无法连接到:' + btn.verify.linkUrl,
        ErrMesg: '',
        status: false
      })
    }
  }

  syncMessageSend = (list) => {
    let param = list.shift()

    if (socket && param) {
      socket.send(JSON.stringify(param))
    }

    if (list && list.length > 0) {
      setTimeout(() => {this.syncMessageSend(list)}, 3000)
    }
  }
  /**
   * @description 操作成功后处理
   * 1、excel导出，成功后取消导出按钮加载中状态
   * 2、状态码为 S 时，显示成功信息后系统默认信息
   * 3、状态码为 -1 时，不显示任何信息
   * 4、模态框执行成功后是否关闭
   * 5、通知主列表刷新
   */
  execSuccess = (res) => {
    const { btn } = this.props

    if (res && res.ErrCode === 'S') { // 执行成功
      notification.success({
        top: 92,
        message: res.ErrMesg || this.state.dict['main.action.confirm.success'],
        duration: btn.verify && btn.verify.stime ? btn.verify.stime : 2
      })
    } else if (res && res.ErrCode === 'Y') { // 执行成功
      Modal.success({
        title: res.ErrMesg || this.state.dict['main.action.confirm.success']
      })
    } else if (res && res.ErrCode === '-1') { // 完成后不提示

    }
    
    this.setState({
      loading: false
    })

    if (btn.execSuccess !== 'never') {
      MKEmitter.emit('refreshByButtonResult', btn.$menuId, btn.execSuccess, btn)
    }
  }

  /**
   * @description 操作失败后处理
   * 1、状态码为 E、N、F、NM 时，显示相应提示信息
   * 2、excel导出，失败后取消导出按钮加载中状态
   * 3、通知主列表刷新
   */
  execError = (res) => {
    const { btn } = this.props

    if (res.ErrCode === 'E') {
      Modal.error({
        title: res.message || res.ErrMesg,
      })
    } else if (res.ErrCode === 'N') {
      notification.error({
        top: 92,
        message: res.message || res.ErrMesg,
        duration: btn.verify && btn.verify.ntime ? btn.verify.ntime : 10
      })
    } else if (res.ErrCode === 'F') {
      notification.error({
        className: 'notification-custom-error',
        top: 92,
        message: res.message || res.ErrMesg,
        duration: btn.verify && btn.verify.ftime ? btn.verify.ftime : 10
      })
    } else if (res.ErrCode === 'NM') {
      message.error(res.message || res.ErrMesg)
    }
    
    this.setState({
      loading: false
    })

    if (btn.execError !== 'never') {
      MKEmitter.emit('refreshByButtonResult', btn.$menuId, btn.execError, btn)
    }
  }

  /**
   * @description 按钮配置信息错误提示
   */
  actionSettingError = () => {
    notification.warning({
      top: 92,
      message: this.state.dict['main.action.settingerror'],
      duration: 5
    })
  }

  handleModelConfig = (config) => {
    let roleId = sessionStorage.getItem('role_id') || '' // 角色ID
    if (config.groups.length > 0) {
      config.groups.forEach(group => {
        group.sublist = group.sublist.map(cell => {
          // 数据源sql语句，预处理， 权限黑名单字段设置为隐藏表单
          if (['select', 'link', 'multiselect', 'radio', 'checkbox', 'checkcard'].includes(cell.type) && cell.resourceType === '1') {
            let _option = Utils.getSelectQueryOptions(cell)

            // 外联数据库替换
            if (window.GLOB.externalDatabase !== null) {
              _option.sql = _option.sql.replace(/@db@/ig, window.GLOB.externalDatabase)
            }

            cell.data_sql = Utils.formatOptions(_option.sql)
            cell.base_sql = window.btoa(window.encodeURIComponent(_option.sql))
            cell.arr_field = _option.field
          }

          // 字段权限黑名单
          if (!cell.blacklist || cell.blacklist.length === 0) return cell
          if (cell.blacklist.filter(v => roleId.indexOf(v) > -1).length > 0) {
            cell.hidden = 'true'
          }

          return cell
        })
      })
    } else {
      config.fields = config.fields.map(cell => {
        // 数据源sql语句，预处理，权限黑名单字段设置为隐藏表单
        if (['select', 'link', 'multiselect', 'radio', 'checkbox', 'checkcard'].includes(cell.type) && cell.resourceType === '1') {
          let _option = Utils.getSelectQueryOptions(cell)

          cell.data_sql = Utils.formatOptions(_option.sql)
          cell.base_sql = window.btoa(window.encodeURIComponent(_option.sql))
          cell.arr_field = _option.field
        }

        // 字段权限黑名单
        if (!cell.blacklist || cell.blacklist.length === 0) return cell
        if (cell.blacklist.filter(v => roleId.indexOf(v) > -1).length > 0) {
          cell.hidden = 'true'
        }

        return cell
      })
    }
    return config
  }

  /**
   * @description 获取按钮配置信息
   */
  improveAction = () => {
    const { btn } = this.props
    const { btnconfig } = this.state

    if (btnconfig) {
      if (btnconfig.setting.display === 'prompt') { // 如果表单以是否框展示
        this.modelconfirm()
      } else {
        this.setState({
          visible: true
        })
      }
    } else {
      Api.getCacheConfig({
        func: 'sPC_Get_LongParam',
        MenuID: btn.uuid
      }).then(res => {
        let _LongParam = ''

        if (res.status && res.LongParam) {
          try {
            _LongParam = JSON.parse(window.decodeURIComponent(window.atob(res.LongParam)))
          } catch (e) {
            console.warn('Parse Failure')
            _LongParam = ''
          }
        }

        if (!res.status) {
          notification.warning({
            top: 92,
            message: res.message,
            duration: 5
          })
          this.updateStatus('over')
        } else if (!_LongParam || (btn.execMode === 'pop' && _LongParam.type !== 'Modal')) {
          notification.warning({
            top: 92,
            message: '未获取到按钮配置信息！',
            duration: 5
          })
          this.updateStatus('over')
        } else {
          _LongParam = this.handleModelConfig(_LongParam)

          this.setState({
            btnconfig: _LongParam
          }, () => {
            if (_LongParam.setting.display === 'prompt') { // 如果表单以是否框展示
              this.modelconfirm()
            } else {
              this.setState({
                visible: true
              })
            }
          })
        }
      })
    }
  }

  /**
   * @description 模态框（表单），确认
   */
  handleOk = () => {
    this.formRef.handleConfirm().then(res => {
      this.setState({
        visible: false
      })
      this.triggerPrint(this.state.tabledata, res)
    })
  }

  /**
   * @description 模态框（表单），取消
   */
  handleCancel = () => {
    this.setState({
      visible: false
    })
    this.updateStatus('over')
  }

  modelconfirm = () => {
    const { BData } = this.props
    const { btnconfig, tabledata } = this.state
    let _this = this
    let _fields = []
    
    if (btnconfig.groups.length > 0) {
      btnconfig.groups.forEach(group => {
        _fields = [..._fields, ...group.sublist]
      })
    } else {
      _fields = btnconfig.fields
    }

    let result = _fields.map(item => {
      let _readin = item.readin !== 'false'
      let _initval = item.initval

      if (item.type === 'linkMain' || item.type === 'funcvar') {
        _readin = false
      }

      if (item.type === 'linkMain' && BData && BData.hasOwnProperty(item.field)) {
        _initval = BData[item.field]
      } else if (_readin && tabledata[0] && tabledata[0].hasOwnProperty(item.field)) {
        _initval = tabledata[0][item.field]
      } else if (item.type === 'date' && _initval) {
        _initval = moment().subtract(_initval, 'days').format('YYYY-MM-DD')
      } else if (item.type === 'datemonth' && _initval) {
        _initval = moment().subtract(_initval, 'month').format('YYYY-MM')
      } else if (item.type === 'datetime' && _initval) {
        _initval = moment().subtract(_initval, 'days').format('YYYY-MM-DD HH:mm:ss')
      }

      let _fieldlen = item.fieldlength || 50
      if (item.type === 'textarea' || item.type === 'fileupload' || item.type === 'multiselect') {
        _fieldlen = item.fieldlength || 512
      } else if (item.type === 'number') {
        _fieldlen = item.decimal ? item.decimal : 0
      }

      if (_initval === undefined) {
        _initval = ''
      }

      return {
        key: item.field,
        readonly: item.readonly === 'true',
        readin: _readin,
        fieldlen: _fieldlen,
        type: item.type,
        value: _initval
      }
    })

    confirm({
      title: this.state.dict['main.action.confirm.tip'],
      onOk() {
        _this.triggerPrint(tabledata, result)
      },
      onCancel() {
        _this.updateStatus('over')
      }
    })
  }

  /**
   * @description 显示模态框
   */
  getModels = () => {
    const { setting, BID, btn } = this.props
    const { btnconfig } = this.state

    if (!this.state.visible || !btnconfig || !btnconfig.setting) return null

    let title = btnconfig.setting.title
    let width = btnconfig.setting.width + 'vw'
    let clickouter = false
    let container = document.body

    if (
      (setting.tabType === 'main' && btnconfig.setting.container === 'tab' && this.props.ContainerId) ||
      (btnconfig.setting.container === 'tab' && btn.ContainerId)
    ) {
      width = btnconfig.setting.width + '%'
      container = () => document.getElementById(this.props.ContainerId || btn.ContainerId)
    }

    if (btnconfig.setting.clickouter === 'close') {
      clickouter = true
    }

    return (
      <Modal
        title={title}
        maskClosable={clickouter}
        getContainer={container}
        wrapClassName='action-modal'
        visible={this.state.visible}
        width={width}
        onOk={this.handleOk}
        onCancel={this.handleCancel}
        destroyOnClose
      >
        <MutilForm
          BID={BID}
          dict={this.state.dict}
          menuType={this.props.menuType}
          action={btnconfig}
          inputSubmit={this.handleOk}
          data={this.state.tabledata[0]}
          BData={this.props.BData}
          wrappedComponentRef={(inst) => this.formRef = inst}
        />
      </Modal>
    )
  }

  render() {
    const { btn, show } = this.props
    const { loadingNumber, loading } = this.state

    if (show === 'actionList') {
      return <div style={{display: 'inline-block'}} onClick={(e) => e.stopPropagation()}>
        <Button
          icon={btn.icon}
          loading={loading}
          className={'mk-btn mk-' + btn.class}
          onClick={() => {this.actionTrigger()}}
        >{loadingNumber ? `(${loadingNumber})` : '' + btn.label}</Button>
        {this.getModels()}
      </div>
    } else { // icon、text、 all 卡片
      return <div style={{display: 'inline-block'}} onClick={(e) => e.stopPropagation()}>
        <Button
          type="link"
          title={show === 'icon' ? btn.label : ''}
          loading={loading}
          style={btn.btnstyle}
          icon={show === 'text' ? '' : (btn.icon || '')}
          onClick={() => {this.actionTrigger()}}
        >{show === 'icon' && btn.icon ? '' : btn.label}</Button>
        {this.getModels()}
      </div>
    }
  }
}

const mapStateToProps = (state) => {
  return {
    tabviews: state.tabviews,
    menuType: state.editLevel
  }
}

const mapDispatchToProps = () => {
  return {}
}

export default connect(mapStateToProps, mapDispatchToProps)(PrintButton)